package ink.wgink.module.activiti.listener.task;

import ink.wgink.exceptions.OaNodeException;
import ink.wgink.interfaces.department.IDepartmentBaseService;
import ink.wgink.interfaces.department.IDepartmentUserBaseService;
import ink.wgink.interfaces.position.IPositionBaseService;
import ink.wgink.interfaces.position.IPositionUserBaseService;
import ink.wgink.interfaces.role.IRoleBaseService;
import ink.wgink.interfaces.role.IRoleUserBaseService;
import ink.wgink.interfaces.user.IUserBaseService;
import ink.wgink.module.activiti.enums.oa.assignee.*;
import ink.wgink.module.activiti.pojo.pos.oa.OaNodeAssigneePO;
import ink.wgink.module.activiti.service.activiti.IActivitiModelService;
import ink.wgink.module.activiti.service.oa.INodeAssigneeService;
import ink.wgink.module.activiti.service.oa.IOaFormReportService;
import ink.wgink.pojo.dtos.department.DepartmentUserDTO;
import ink.wgink.pojo.dtos.position.PositionDTO;
import ink.wgink.pojo.dtos.user.UserDTO;
import ink.wgink.util.ArrayListUtil;
import org.activiti.bpmn.model.UserTask;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.activiti.engine.delegate.JavaDelegate;
import org.activiti.engine.delegate.TaskListener;
import org.activiti.engine.history.HistoricTaskInstance;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

/**
 * @ClassName: OaUserTaskCompleteListener
 * @Description: OA用户任务节点完成监听器
 * @Author: wanggeng
 * @Date: 2022/3/28 10:31
 * @Version: 1.0
 */
@Component
public class OaUserTaskCompleteListener implements TaskListener, JavaDelegate {

    private static final Logger LOG = LoggerFactory.getLogger(OaUserTaskCompleteListener.class);
    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private RepositoryService repositoryService;
    @Autowired
    private IActivitiModelService activitiModelService;
    @Autowired
    private INodeAssigneeService nodeAssigneeService;
    @Autowired
    private IOaFormReportService oaFormReportService;
    @Autowired
    private IUserBaseService userBaseService;
    @Autowired
    private IDepartmentBaseService departmentBaseService;
    @Autowired
    private IDepartmentUserBaseService departmentUserBaseService;
    @Autowired
    private IRoleBaseService roleBaseService;
    @Autowired
    private IRoleUserBaseService roleUserBaseService;
    @Autowired
    private IPositionBaseService positionBaseService;
    @Autowired
    private IPositionUserBaseService positionUserBaseService;

    @Override
    public void notify(DelegateTask delegateTask) {
        LOG.debug(">>>>>> 代理人任务完成 <<<<<<");
        /**
         String processDefinitionId = delegateTask.getProcessDefinitionId();
         String processInstanceId = delegateTask.getProcessInstanceId();
         LOG.debug("查找流程定义");
         ProcessDefinition processDefinition = repositoryService.getProcessDefinition(processDefinitionId);
         String deploymentId = processDefinition.getDeploymentId();

         LOG.debug("清空流程变量中的 代理人列表");
         delegateTask.removeVariable(IActivitiModelService.ASSIGNEE_LIST);

         Map<String, Object> variables = delegateTask.getVariablesLocal();
         LOG.debug("查询表单数据：variables: {}", variables);
         LOG.debug("获取下一节点");
         UserTask nextUserTask = activitiModelService.getNextUserTask(delegateTask.getTaskDefinitionKey(), delegateTask.getProcessDefinitionId(), variables);
         if (nextUserTask == null) {
         LOG.debug("下一个节点不存在");
         return;
         }
         LOG.debug("获取节点代理人配置");
         OaNodeAssigneePO oaNodeAssigneePO = nodeAssigneeService.getPOByDeploymentIdAndNodeId(deploymentId, nextUserTask.getId());
         if (oaNodeAssigneePO == null) {
         throw new OaNodeException("下一节点没有配置代理人信息");
         }
         List<String> assigneeUserIds = nodeAssigneeService.listAssigneeUserId(processInstanceId, delegateTask.getAssignee(), oaNodeAssigneePO);
         delegateTask.setVariable(IActivitiModelService.ASSIGNEE_LIST, assigneeUserIds);
         **/
    }

    @Override
    public void execute(DelegateExecution execution) {

    }

    /**
     * @param userTask
     * @param oaNodeAssigneePO
     */
    private void normal(DelegateTask delegateTask, UserTask userTask, OaNodeAssigneePO oaNodeAssigneePO) {
        if (StringUtils.equals(oaNodeAssigneePO.getAssigneeType(), AssigneeTypeEnum.APPOINT.getValue())) {
            LOG.debug("assigneeType -> appoint");
            appoint(delegateTask, userTask, oaNodeAssigneePO);
            return;
        }
        if (StringUtils.equals(oaNodeAssigneePO.getAssigneeType(), AssigneeTypeEnum.AUTO.getValue())) {
            LOG.debug("assigneeType -> auto");
            auto(delegateTask, userTask, oaNodeAssigneePO, NodeTypeEnum.NORMAL);
            return;
        }
    }

    /**
     * 设置多实例节点
     *
     * @param delegateTask
     * @param userTask
     * @param oaNodeAssigneePO
     */
    private void multiple(DelegateTask delegateTask, UserTask userTask, OaNodeAssigneePO oaNodeAssigneePO) {
        if (StringUtils.equals(oaNodeAssigneePO.getAssigneeType(), AssigneeTypeEnum.APPOINT.getValue())) {
            LOG.debug("assigneeType -> appoint");
            String multipleAssignees = oaNodeAssigneePO.getMultipleAssignees();
            if (StringUtils.isBlank(multipleAssignees)) {
                throw new OaNodeException("多实例代理人未设置");
            }
            List<UserDTO> userDTOs = userBaseService.listByUserIds(Arrays.asList(multipleAssignees.split(",")));
            if (userDTOs.isEmpty()) {
                throw new OaNodeException("多实例代理人不存在");
            }
            List<String> existUserIds = ArrayListUtil.listBeanStringIdValue(userDTOs, "userId", UserDTO.class);
            delegateTask.setVariable(IActivitiModelService.ASSIGNEE_LIST, existUserIds);
            return;
        }
        if (StringUtils.equals(oaNodeAssigneePO.getAssigneeType(), AssigneeTypeEnum.AUTO.getValue())) {
            LOG.debug("assigneeType -> auto");
            auto(delegateTask, userTask, oaNodeAssigneePO, NodeTypeEnum.MULTIPLE);
            return;
        }
    }

    /**
     * 手动指定
     *
     * @param delegateTask
     * @param userTask
     * @param oaNodeAssigneePO
     */
    private void appoint(DelegateTask delegateTask, UserTask userTask, OaNodeAssigneePO oaNodeAssigneePO) {
        if (StringUtils.equals(oaNodeAssigneePO.getAssigneeCount(), AssigneeCountEnum.SINGLE.getValue())) {
            LOG.debug("assigneeCount -> single");
            String assignee = oaNodeAssigneePO.getAssignee();
            if (StringUtils.isBlank(assignee)) {
                throw new OaNodeException("下节点代理人未设置");
            }
            UserDTO userDTO = userBaseService.get(assignee);
            if (userDTO == null) {
                throw new OaNodeException("下节点代理人不存在");
            }
            LOG.debug("设置普通节点代理人");
            userTask.setAssignee(userDTO.getUserId());
            return;
        }
        if (StringUtils.equals(oaNodeAssigneePO.getAssigneeCount(), AssigneeCountEnum.CANDIDATE.getValue())) {
            LOG.debug("assigneeCount -> candidate");
            String candidates = oaNodeAssigneePO.getCandidates();
            if (StringUtils.isBlank(candidates)) {
                throw new OaNodeException("下节点候选人未设置");
            }
            setNextUserTaskAssignees(delegateTask, userTask, Arrays.asList(candidates.split(",")), NodeTypeEnum.NORMAL);
            return;
        }
    }

    /**
     * 自动获取
     *
     * @param delegateTask
     * @param userTask
     * @param oaNodeAssigneePO
     */
    private void auto(DelegateTask delegateTask, UserTask userTask, OaNodeAssigneePO oaNodeAssigneePO, NodeTypeEnum nodeType) {
        if (StringUtils.equals(oaNodeAssigneePO.getAutoAssignType(), AutoAssignTypeEnum.SCOPE.getValue())) {
            LOG.debug("autoAssigneeType -> scope");
            departmentType(delegateTask, userTask, oaNodeAssigneePO, nodeType);
            return;
        }
        if (StringUtils.equals(oaNodeAssigneePO.getAutoAssignType(), AutoAssignTypeEnum.QUICK.getValue())) {
            LOG.debug("autoAssigneeType -> quick");
            quickAssignee(delegateTask, userTask, oaNodeAssigneePO, nodeType);
            return;
        }
    }

    /**
     * 快捷指定
     *
     * @param delegateTask
     * @param userTask
     * @param oaNodeAssigneePO
     * @param nodeType
     */
    private void quickAssignee(DelegateTask delegateTask, UserTask userTask, OaNodeAssigneePO oaNodeAssigneePO, NodeTypeEnum nodeType) {
        String quickAssignee = oaNodeAssigneePO.getQuickAssignee();
        if (StringUtils.equals(quickAssignee, QuickAssigneeEnum.FIRST_STEP_ASSIGN.getValue())) {
            LOG.debug("quickAssignee -> firstStepAssign:第一步代理人");
            HistoricTaskInstance startTask = activitiModelService.getStartTask(delegateTask.getProcessInstanceId());
            userTask.setAssignee(startTask.getAssignee());
            return;
        }
        if (StringUtils.equals(quickAssignee, QuickAssigneeEnum.FIRST_STEP_SUPERIOR_POSITION.getValue())) {
            starterSuperiorPosition(delegateTask, userTask, oaNodeAssigneePO, nodeType);
            return;
        }
        if (StringUtils.equals(quickAssignee, QuickAssigneeEnum.PREVIOUS_STEP_ASSIGNEE_SUBORDINATE_POSITION.getValue())) {
            previousStepAssigneeSubordinatePosition(delegateTask, userTask, oaNodeAssigneePO, nodeType);
            return;
        }
        if (StringUtils.equals(quickAssignee, QuickAssigneeEnum.PREVIOUS_STEP_ASSIGNEE_SUPERIOR_POSITION.getValue())) {
            previousStepAssigneeSuperiorPosition(delegateTask, userTask, oaNodeAssigneePO, nodeType);
            return;
        }
        if (StringUtils.equals(quickAssignee, QuickAssigneeEnum.PREVIOUS_STEP_ASSIGNEE_SUPERIOR_POSITION.getValue())) {
            previousStepAssigneeSameDepartment(delegateTask, userTask, oaNodeAssigneePO, nodeType);
            return;
        }
        if (StringUtils.equals(quickAssignee, QuickAssigneeEnum.PREVIOUS_N_STEP_ASSIGNEE_SUBORDINATE_POSITION.getValue())) {
            previousNStepAssigneeSubordinatePosition(delegateTask, userTask, oaNodeAssigneePO, nodeType);
            return;
        }
    }

    /**
     * 发起人上级岗位
     *
     * @param delegateTask
     * @param userTask
     * @param oaNodeAssigneePO
     * @param nodeType
     */
    private void starterSuperiorPosition(DelegateTask delegateTask, UserTask userTask, OaNodeAssigneePO oaNodeAssigneePO, NodeTypeEnum nodeType) {
        // 同部门
        LOG.debug("quickAssignee -> starterSuperiorPosition:发起人上级岗位");
        // 开始节点代理人
        HistoricTaskInstance startTask = activitiModelService.getStartTask(delegateTask.getProcessInstanceId());
        String starterUserId = startTask.getAssignee();
        LOG.debug("查询同部门下所有用户");
        List<String> departmentUserIds = departmentUserBaseService.listSameDepartmentUserIdByUserId(starterUserId);
        List<String> positionIds = positionUserBaseService.listPositionIdByUserId(starterUserId);
        List<PositionDTO> parentPositionDTOs = positionBaseService.listParentByPositionIds(positionIds);
        if (parentPositionDTOs.isEmpty()) {
            throw new OaNodeException("发起人不存在上级岗位");
        }
        List<String> parentPositionIds = ArrayListUtil.listBeanStringIdValue(parentPositionDTOs, "positionId", PositionDTO.class);
        LOG.debug("查询发起人同部门上级岗位用户ID列表");
        List<String> parentPositionUserIds = positionUserBaseService.listUserIdByPositionIdsAndUserIds(parentPositionIds, departmentUserIds);
        LOG.debug("查询发起人同部门上级岗位用户列表");
        List<UserDTO> parentPositionUserDTOs = userBaseService.listByUserIds(parentPositionUserIds);
        if (parentPositionUserDTOs.isEmpty()) {
            throw new OaNodeException("发起人上级岗位没有用户");
        }
        // 上级岗位可以有自己
        List<String> existParentPositionUserIds = ArrayListUtil.listBeanStringIdValue(parentPositionUserDTOs, "userId", UserDTO.class);
        if (NodeTypeEnum.NORMAL.equals(nodeType)) {
            LOG.debug("候选人用户列表");
            userTask.setCandidateUsers(existParentPositionUserIds);
        } else {
            delegateTask.setVariable(IActivitiModelService.ASSIGNEE_LIST, existParentPositionUserIds);
        }
    }

    /**
     * 上一步代理人下级岗位
     *
     * @param delegateTask
     * @param userTask
     * @param oaNodeAssigneePO
     * @param nodeType
     */
    private void previousStepAssigneeSubordinatePosition(DelegateTask delegateTask, UserTask userTask, OaNodeAssigneePO oaNodeAssigneePO, NodeTypeEnum nodeType) {
        // 同部门
        LOG.debug("quickAssignee -> previousStepAssigneeSubordinatePosition:上一步代理人下级岗位");
        // 当前任务人
        String assignee = delegateTask.getAssignee();
        LOG.debug("查询当前代理人同部门下所有用户");
        List<String> departmentUserIds = departmentUserBaseService.listSameDepartmentUserIdByUserId(assignee);
        LOG.debug("查询当前代理人的岗位");
        List<String> positionIds = positionUserBaseService.listPositionIdByUserId(assignee);
        LOG.debug("查询下级岗位列表");
        List<PositionDTO> childPositionDTOs = positionBaseService.listByPositionParentIds(positionIds);
        if (childPositionDTOs.isEmpty()) {
            throw new OaNodeException("当前代理人没有下级岗位");
        }
        List<String> childPositionIds = ArrayListUtil.listBeanStringIdValue(childPositionDTOs, "positionId", PositionDTO.class);
        LOG.debug("查询下级岗位的用户ID列表");
        List<String> childPositionUserIds = positionUserBaseService.listUserIdByPositionIdsAndUserIds(childPositionIds, departmentUserIds);
        LOG.debug("查询下级岗位用户列表");
        List<UserDTO> childPositionUserDTOs = userBaseService.listByUserIds(childPositionUserIds);
        if (childPositionUserDTOs.isEmpty()) {
            throw new OaNodeException("下级岗位没有用户");
        }
        List<String> existChildPositionUserIds = ArrayListUtil.listBeanStringIdValue(childPositionUserDTOs, "userId", UserDTO.class);
        if (NodeTypeEnum.NORMAL.equals(nodeType)) {
            LOG.debug("设置候选人列表");
            userTask.setCandidateUsers(existChildPositionUserIds);
        } else {
            delegateTask.setVariable(IActivitiModelService.ASSIGNEE_LIST, existChildPositionUserIds);
        }
    }

    /**
     * 上一步代理人上级岗位
     *
     * @param delegateTask
     * @param userTask
     * @param oaNodeAssigneePO
     * @param nodeType
     */
    private void previousStepAssigneeSuperiorPosition(DelegateTask delegateTask, UserTask userTask, OaNodeAssigneePO oaNodeAssigneePO, NodeTypeEnum nodeType) {
        // 同部门
        LOG.debug("quickAssignee -> previousStepAssigneeSuperiorPosition:上一步代理人上级岗位");
        // 当前代理人
        String assignee = delegateTask.getAssignee();
        LOG.debug("查询当前代理人同部门下所有用户");
        List<String> departmentUserIds = departmentUserBaseService.listSameDepartmentUserIdByUserId(assignee);
        LOG.debug("查询当前代理人的岗位");
        List<String> positionIds = positionUserBaseService.listPositionIdByUserId(assignee);
        LOG.debug("查询上级岗位列表");
        List<PositionDTO> parentPositionDTOs = positionBaseService.listParentByPositionIds(positionIds);
        if (parentPositionDTOs.isEmpty()) {
            throw new OaNodeException("当前代理人没有上级岗位");
        }
        List<String> parentPositionIds = ArrayListUtil.listBeanStringIdValue(parentPositionDTOs, "positionId", PositionDTO.class);
        LOG.debug("查询上级岗位的用户ID列表");
        List<String> parentPositionUserIds = positionUserBaseService.listUserIdByPositionIdsAndUserIds(parentPositionIds, departmentUserIds);
        LOG.debug("查询上级岗位用户列表");
        List<UserDTO> parentPositionUserDTOs = userBaseService.listByUserIds(parentPositionUserIds);
        if (parentPositionUserDTOs.isEmpty()) {
            throw new OaNodeException("上级岗位没有用户");
        }
        List<String> existParentPositionUserIds = ArrayListUtil.listBeanStringIdValue(parentPositionUserDTOs, "userId", UserDTO.class);
        if (NodeTypeEnum.NORMAL.equals(nodeType)) {
            LOG.debug("设置候选人列表");
            userTask.setCandidateUsers(existParentPositionUserIds);
        } else {
            delegateTask.setVariable(IActivitiModelService.ASSIGNEE_LIST, existParentPositionUserIds);
        }
    }

    /**
     * 上一步代理人同部门
     *
     * @param delegateTask
     * @param userTask
     * @param oaNodeAssigneePO
     * @param nodeType
     */
    private void previousStepAssigneeSameDepartment(DelegateTask delegateTask, UserTask userTask, OaNodeAssigneePO oaNodeAssigneePO, NodeTypeEnum nodeType) {
        // 同部门
        LOG.debug("quickAssignee -> previousStepAssigneeSameDepartment:上一步代理人同部门");
        String assignee = delegateTask.getAssignee();
        LOG.debug("查询当前代理人同部门下所有用户");
        List<String> departmentUserIds = departmentUserBaseService.listSameDepartmentUserIdByUserId(assignee);
        LOG.debug("排除自己");
        clearUserIdsCurrentTaskAssignee(departmentUserIds, assignee);
        List<UserDTO> departmentUserDTOs = userBaseService.listByUserIds(departmentUserIds);
        if (departmentUserDTOs.isEmpty()) {
            throw new OaNodeException("同部门下没有其他用户");
        }
        List<String> existChildPositionUserIds = ArrayListUtil.listBeanStringIdValue(departmentUserDTOs, "userId", UserDTO.class);
        if (NodeTypeEnum.NORMAL.equals(nodeType)) {
            LOG.debug("设置候选人列表");
            userTask.setCandidateUsers(existChildPositionUserIds);
        } else {
            delegateTask.setVariable(IActivitiModelService.ASSIGNEE_LIST, existChildPositionUserIds);
        }
    }

    /**
     * 前 N 步代理人
     *
     * @param delegateTask
     * @param userTask
     * @param oaNodeAssigneePO
     * @param nodeType
     */
    private void previousNStepAssigneeSubordinatePosition(DelegateTask delegateTask, UserTask userTask, OaNodeAssigneePO oaNodeAssigneePO, NodeTypeEnum nodeType) {
        LOG.debug("quickAssignee -> previousNStepAssigneeSubordinatePosition:前N步代理人");
        Integer previousNStep = oaNodeAssigneePO.getPreviousNStep();
        HistoricTaskInstance prevTask = activitiModelService.getPrevTask(delegateTask.getProcessInstanceId(), previousNStep - 1);
        String assignee = prevTask.getAssignee();
        if (NodeTypeEnum.NORMAL.equals(nodeType)) {
            userTask.setAssignee(assignee);
        } else {
            delegateTask.setVariable(IActivitiModelService.ASSIGNEE_LIST, Arrays.asList(assignee));
        }
    }

    /**
     * 部门类型
     *
     * @param delegateTask
     * @param userTask
     * @param oaNodeAssigneePO
     * @param nodeType
     */
    private void departmentType(DelegateTask delegateTask, UserTask userTask, OaNodeAssigneePO oaNodeAssigneePO, NodeTypeEnum nodeType) {
        if (StringUtils.equals(oaNodeAssigneePO.getDepartmentType(), DepartmentTypeEnum.APPOINT.getValue())) {
            LOG.debug("departmentType -> appoint");
            String departmentIds = oaNodeAssigneePO.getDepartments();
            String roleIds = oaNodeAssigneePO.getRoles();
            String positionIds = oaNodeAssigneePO.getPositions();
            if (StringUtils.isBlank(departmentIds) && StringUtils.isBlank(roleIds) && StringUtils.isBlank(positionIds)) {
                throw new OaNodeException("下节点没有设置部门、角色、职位");
            }
            if (!StringUtils.isBlank(departmentIds)) {
                LOG.debug("查询部门用户");
                List<String> departmentUserIds = departmentUserBaseService.listUserId(Arrays.asList(departmentIds.split(",")));
                setDepartmentCandidateUser(delegateTask, userTask, departmentUserIds, roleIds, positionIds, nodeType);
                return;
            }
            if (!StringUtils.isBlank(roleIds)) {
                LOG.debug("查询角色用户");
                List<String> roleUserIds = roleUserBaseService.listUserId(Arrays.asList(roleIds.split(",")));
                setRoleCandidateUser(delegateTask, userTask, roleUserIds, positionIds, nodeType);
                return;
            }
            LOG.debug("查询职位用户");
            List<String> positionUserIds = positionUserBaseService.listUserId(Arrays.asList(positionIds.split(",")));
            setPositionCandidateUser(delegateTask, userTask, positionUserIds, nodeType);
            return;
        }
        if (StringUtils.equals(oaNodeAssigneePO.getDepartmentType(), DepartmentTypeEnum.BELONG.getValue())) {
            LOG.debug("departmentType -> belong");
            String roleIds = oaNodeAssigneePO.getRoles();
            String positionIds = oaNodeAssigneePO.getPositions();
            List<DepartmentUserDTO> departmentUserDTOs = departmentUserBaseService.listByUserIds(Arrays.asList(delegateTask.getAssignee()));
            List<String> departmentIds = ArrayListUtil.listBeanStringIdValue(departmentUserDTOs, "departmentId", DepartmentUserDTO.class);
            if (departmentIds.isEmpty()) {
                throw new OaNodeException("代理人不存在部门");
            }
            List<String> departmentUserIds = departmentUserBaseService.listUserId(departmentIds);
            setDepartmentCandidateUser(delegateTask, userTask, departmentUserIds, roleIds, positionIds, nodeType);
        }
    }

    /**
     * 设置部门候选用户
     *
     * @param delegateTask
     * @param userTask
     * @param departmentUserIds
     * @param roleIds
     * @param positionIds
     * @param nodeType
     */
    private void setDepartmentCandidateUser(DelegateTask delegateTask, UserTask userTask, List<String> departmentUserIds, String roleIds, String positionIds, NodeTypeEnum nodeType) {
        if (!StringUtils.isBlank(roleIds)) {
            LOG.debug("查询角色用户");
            List<String> roleUserIds = roleUserBaseService.listUserIdByRoleIdsAndUserIds(Arrays.asList(roleIds.split(",")), departmentUserIds);
            setRoleCandidateUser(delegateTask, userTask, roleUserIds, positionIds, nodeType);
            return;
        }
        setNextUserTaskAssignees(delegateTask, userTask, departmentUserIds, nodeType);
    }

    /**
     * 设置角色候选用户
     *
     * @param delegateTask
     * @param userTask
     * @param roleUserIds
     * @param positionIds
     * @param nodeType
     */
    private void setRoleCandidateUser(DelegateTask delegateTask, UserTask userTask, List<String> roleUserIds, String positionIds, NodeTypeEnum nodeType) {
        if (!StringUtils.isBlank(positionIds)) {
            LOG.debug("查询职位用户");
            List<String> positionUserIds = positionUserBaseService.listUserIdByPositionIdsAndUserIds(Arrays.asList(positionIds.split(",")), roleUserIds);
            setPositionCandidateUser(delegateTask, userTask, positionUserIds, nodeType);
            return;
        }
        setNextUserTaskAssignees(delegateTask, userTask, roleUserIds, nodeType);
    }

    /**
     * 设置职位候选用户
     *
     * @param delegateTask
     * @param userTask
     * @param positionUserIds
     * @param nodeType
     */
    private void setPositionCandidateUser(DelegateTask delegateTask, UserTask userTask, List<String> positionUserIds, NodeTypeEnum nodeType) {
        setNextUserTaskAssignees(delegateTask, userTask, positionUserIds, nodeType);
    }

    /**
     * 设置下个用户任务代理人清单
     *
     * @param delegateTask
     * @param userTask
     * @param userIds
     * @param nodeType
     */
    private void setNextUserTaskAssignees(DelegateTask delegateTask, UserTask userTask, List<String> userIds, NodeTypeEnum nodeType) {
        List<UserDTO> userDTOs = userBaseService.listByUserIds(userIds);
        if (userDTOs.isEmpty()) {
            throw new OaNodeException("下节点候选人不存在");
        }
        LOG.debug("设置候选人");
        List<String> existUserIds = ArrayListUtil.listBeanStringIdValue(userDTOs, "userId", UserDTO.class);
        clearUserIdsCurrentTaskAssignee(existUserIds, delegateTask.getAssignee());
        delegateTask.setVariable(IActivitiModelService.ASSIGNEE_LIST, existUserIds);
    }

    /**
     * 清除用户列表中当前任务代理人
     *
     * @param userIds
     * @param assignee
     */
    private void clearUserIdsCurrentTaskAssignee(List<String> userIds, String assignee) {
        for (String userId : userIds) {
            if (StringUtils.equals(userId, assignee)) {
                userIds.remove(userId);
                break;
            }
        }
    }

}
